#include <windows.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>

#pragma comment(lib,"sdl.lib")
#pragma comment(lib,"sdlmain.lib")
#pragma comment(lib, "opengl32.lib" )
#pragma comment(lib, "glu32.lib" )

#define XRESOL 800
#define YRESOL 600

int initGL()
{
	// init
	glShadeModel(GL_FLAT);
	glClearColor(0.0f,0.0f,0.0f,0.0f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
	// cadrage
	glViewport(0,0,XRESOL,YRESOL);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,4.0/3.0,0.1f,100.0f);
	glEnable(GL_TEXTURE_2D);
	return 1;
}

int KeyControl(float* pe)
{
	Uint8* ks = SDL_GetKeyState(NULL);
	SDL_PumpEvents();
	if (ks[SDLK_ESCAPE])
		return 1;
	if (ks[SDLK_UP])
		(*pe)+=0.01f;
	if (ks[SDLK_DOWN])
		(*pe)-=0.01f;
	return 0;
}

int Quad(float x,float y,float z,GLuint texture)
{
	glBindTexture(GL_TEXTURE_2D,texture);
	glBegin(GL_QUADS);               
		glTexCoord2f(0.0,1.0);glVertex3f(x,y,z);
		glTexCoord2f(1.0,1.0);glVertex3f(x+0.99,y,z);
		glTexCoord2f(1.0,0.0);glVertex3f(x+0.99,y+0.99,z);
		glTexCoord2f(0.0,0.0);glVertex3f(x,y+0.99,z);
	glEnd();  
	return 0;
}


GLuint ParametrerTexture(SDL_Surface* tex32,int mipmap,GLint min,GLint max)
{
	GLuint res;
	glGenTextures(1,&res);
	glBindTexture(GL_TEXTURE_2D,res);
	if (mipmap)
		gluBuild2DMipmaps(GL_TEXTURE_2D,4, tex32->w, tex32->h,GL_BGRA_EXT,GL_UNSIGNED_BYTE,tex32->pixels);
	else
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGB, tex32->w, tex32->h,0,GL_BGRA_EXT, GL_UNSIGNED_BYTE, tex32->pixels);
	glTexParameteri( GL_TEXTURE_2D , GL_TEXTURE_MIN_FILTER,min); 
	glTexParameteri( GL_TEXTURE_2D , GL_TEXTURE_MAG_FILTER,max); 
	return res;
}

int InitTextures(GLuint textures[8])
{
	SDL_Surface* tex = SDL_LoadBMP("mur.bmp");
	SDL_Surface* tex32 = SDL_CreateRGBSurface(SDL_SWSURFACE,tex->w,tex->h,32,0,0,0,0);
	SDL_BlitSurface(tex,NULL,tex32,NULL);
	SDL_FreeSurface(tex);
// bord gauche
	textures[0] = ParametrerTexture(tex32,0,GL_NEAREST,GL_NEAREST);
	textures[4] = ParametrerTexture(tex32,1,GL_NEAREST,GL_NEAREST);
// bord droit
	textures[3] = ParametrerTexture(tex32,0,GL_LINEAR,GL_LINEAR);
	textures[7] = ParametrerTexture(tex32,1,GL_LINEAR,GL_LINEAR);
// milieu
	textures[1] = ParametrerTexture(tex32,1,GL_NEAREST_MIPMAP_NEAREST,GL_NEAREST);
	textures[2] = ParametrerTexture(tex32,1,GL_NEAREST_MIPMAP_LINEAR,GL_NEAREST);
	textures[5] = ParametrerTexture(tex32,1,GL_LINEAR_MIPMAP_NEAREST,GL_LINEAR);
	textures[6] = ParametrerTexture(tex32,1,GL_LINEAR_MIPMAP_LINEAR,GL_LINEAR);
	SDL_FreeSurface(tex32);
	return 0;
}

int main(int argc, char **argv)
{
	GLuint textures[8];
	SDL_Surface *surface;
	float e = 0.0f;
	SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE);
	surface = SDL_SetVideoMode(XRESOL,YRESOL,32,SDL_HWSURFACE|SDL_OPENGL);
	initGL();
	InitTextures(textures);
	while(!KeyControl(&e))
	{
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
		glMatrixMode( GL_MODELVIEW );
		glLoadIdentity();
		gluLookAt(0,0,5,0,0,4,0,1,0);

		Quad(-2,0,e,textures[0]);
		Quad(-1,0,e,textures[1]);
		Quad(0,0,e,textures[2]);
		Quad(1,0,e,textures[3]);
		Quad(-2,-1,e,textures[4]);
		Quad(-1,-1,e,textures[5]);
		Quad(0,-1,e,textures[6]);
		Quad(1,-1,e,textures[7]);

		SDL_GL_SwapBuffers( );
	}
	SDL_Quit();
	return 0;
}

/*!F
	Ici, je vais faire des tests de filtrage de texture.

	Je ne vais pas trop parler d'initialisation de texture :
	glEnable(GL_TEXTURE_2D);
	glGenTextures,glBindTexture et glTexCoord2f

	Mais surtout des diffrents filtrages que j'applique.

	Je cre un tableau de 8 textures :
	GLuint textures[8];

	Je les initialise dans InitTextures, et je les affiche dans un Quad.
	Les flches permettent de zoomer sur les textures.

	Dans la fonction InitTextures, je charge une image avec SDL, je la convertis en 32 bits, 
	puis je cre 8 texture avec ma fonction ParametrerTexture,  laquelle je vais donner diffrents paramtres.

	Et c'est la que tout va se jouer.

// Mipmapping

	Il y a deux faons de crer des textures, sans mipmapping (avec glTexImage2D), et avec mipmapping (gluBuild2DMipmaps)

	Le mipmapping, c'est prcalculer des images 4 fois plus petites rcursivement, qui serviront  faire un meilleur lissage en
	cas de dzoom. Le mipmapping prendra environ 1/3 de mmoire (VRAM) en plus.

	http://en.wikipedia.org/wiki/Mipmap

	L'utilit sera d'afficher une image ou une autre (l'image principale ou une des images plus petites) en fonction de la distance
	de la texture  la camra.

	Lanons le programme, et regardons tout a : 
	Il y a 8 carreaux texturs, que nous dsigneront selon l'ordre de lecture naturelle.

	Nous allons tout d'abord commencer par voir ce qu'il se passe quand on zoom. Zommez avec les flches.
	Vous pouvez constater en zoomant  fond que les carreaux d'en haut sont pixelis tandis que ceux d'en bas sont lisss.

	Dans ma fonction ParametrerTexture, c'est le dernier paramtre qui dtermine le type de zoom.
	Deux valeurs possibles :
	
	GL_NEAREST : pixelisation
	GL_LINEAR  : lissage (interpolation bilinaire)

	Maintenant, voyons ce qu'il se passe si on dzoom : et la, c'est l'avant dernier paramtre qui jouera.
	Dzoomez  fond, et constatez : 
	
	- d'abord, a scintille immdiatement sur les surfaces 1 et 5 (celles de gauche)

	Leur point commun est GL_NEAREST pour le min : le scintillement est immdiat.
	Leur diffrence, l'une est avec et l'autre sans mipmapping. Cependant, dans la mesure ou on n'utilise pas d'options de mipmapping,
	ce sera la mme chose.

	Si vous continuez  dzoomer, vous verrez qu'au bout d'un moment, le rendu des surfaces 4 et 8 (de droite) va devenir crado.
	Si vous continuez  dzoomer  fond, les traits blancs de la brique vont "clignotter"

	Les paramtres des surfaces de droite sont GL_LINEAR : dans les deux cas, on fait un lissage bilinaire, mais en s'appuyant sur la texture
	principale : la grande texture de base, l'image de 512*512. Et donc ce sont des erreurs d'arrondis qui font cet effet, du fait que la 
	texture affiche est beaucoup beaucoup plus petite que l'image d'origine.

	Tout a, c'est sans se servir du mipapping. Mme si les surfaces du bas (5 et 8) ont t cres avec le mipmapping, les options 
	de zoom GL_NEAREST et GL_LINEAR font qu'on ne s'en sert pas.

	Voyons maintenant le carr central ou on s'en sert :

	Zoomez, et dzoomez progressivement, vous allez voir un moment une cassure sur les carrs 2 et 6. (gauche milieu).
	a va "devenir flou" d'un seul coup.

	Les paramtres utiliss sont :

	GL_NEAREST_MIPMAP_NEAREST et GL_LINEAR_MIPMAP_NEAREST

	C'est le NEAREST de la fin qu'ils ont en commun, il veut dire "utilise la mipmap la plus proche en fonction de l'loignement), donc 
	il y a une cassure, un seuil au dela duquel on utilise une mipmap ou une autre. (une image, ou une plus petite)

	Leur diffrence est le NEAREST ou LINEAR (aprs le GL)

	Cela veut dire qu'une fois qu'on a choisi le MIPMAP, soit on prend le pixel le plus proche (cas en haut), soit on fait 
	une interpollation bilinaire (cas en bas). 
	Outre la cassure, le rsultat en bas est plus lisse.

	Voyons maintenant les deux derniers cas, les surfaces 3 et 7 (au milieu  droite).

	Leurs options sont les suivantes :

	GL_NEAREST_MIPMAP_LINEAR et GL_LINEAR_MIPMAP_LINEAR

	Cette fois ci, c'est le LINEAR de la fin qu'ils ont en commun, il veut dire "fait un lissage entre 2 mipmaps" : 
	au lieu d'en choisir brutalement une ou l'autre en fonction de la distance, on choisit les deux les plus proches,
	et on fera une interpollation. 

	De ce fait, il n'y a plus de cassure en zoomant et dzoomant.

	Il y a juste celui d'en haut qui scintille un peu, parce qu'il prendra les pixels les plus proches dans chaque texture.

	Celui d'en bas, lui, interpolle bilinaire chacune des deux mipmap, puis interpolle en fonction des 2 mipmap les plus proches.

	C'est une interpollation trilinaire.

	Le gagnant, enfin le plus doux, est bien sur la surface numro 7 !
!*/

/*!E
	No explanations yet.
!*/

/*
titre: Test de mipmapping.
titreE: Mipmapping.
meta: C;OpenGL
metaE: C;OpenGL
desc : Test de mipmapping.
descE: Mipmapping.
cle  : gluBuild2DMipmaps,glTexParameteri
linkA: mur.bmp
parag: 5.2. Textures.
paragE: 5.2. Textures.
*/

